#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Messages relating to signatures
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/signing.yaml with generate.py.
# Please do not hand edit!


class UtcTime(object):
  """UtcTime.
  
  
  Parameters
  ----------
  year : int
    Year
  month : int
    Month (range 1 .. 12)
  day : int
    days in the month (range 1-31)
  hours : int
    hours of day (range 0-23)
  minutes : int
    minutes of hour (range 0-59)
  seconds : int
    seconds of minute (range 0-60) rounded down
  ns : int
    nanoseconds of second (range 0-999999999)

  """
  _parser = construct.Struct(
                     'year' / construct.Int16ul,
                     'month' / construct.Int8ul,
                     'day' / construct.Int8ul,
                     'hours' / construct.Int8ul,
                     'minutes' / construct.Int8ul,
                     'seconds' / construct.Int8ul,
                     'ns' / construct.Int32ul,)
  __slots__ = [
               'year',
               'month',
               'day',
               'hours',
               'minutes',
               'seconds',
               'ns',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.year = kwargs.pop('year')
      self.month = kwargs.pop('month')
      self.day = kwargs.pop('day')
      self.hours = kwargs.pop('hours')
      self.minutes = kwargs.pop('minutes')
      self.seconds = kwargs.pop('seconds')
      self.ns = kwargs.pop('ns')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = UtcTime._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class ECDSASignature(object):
  """ECDSASignature.
  
  
  Parameters
  ----------
  len : int
    Number of bytes to use of the signature field.  The DER encoded signature
    has a maximum size of 72 bytes but can vary between 70 and 72 bytes in
    length.
  data : array
    DER encoded ECDSA signature for the messages using SHA-256 as the digest
    algorithm.

  """
  _parser = construct.Struct(
                     'len' / construct.Int8ul,
                     'data' / construct.Array(72, construct.Int8ul),)
  __slots__ = [
               'len',
               'data',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.len = kwargs.pop('len')
      self.data = kwargs.pop('data')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = ECDSASignature._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
SBP_MSG_ECDSA_CERTIFICATE = 0x0C04
class MsgEcdsaCertificate(SBP):
  """SBP class for message MSG_ECDSA_CERTIFICATE (0x0C04).

  You can have MSG_ECDSA_CERTIFICATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  A DER encoded x.509 ECDSA-256 certificate (using curve secp256r1).

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  n_msg : int
    Total number messages that make up the certificate. The first nibble (mask
    0xF0 or left shifted by 4 bits) is the size of the sequence (n), second
    nibble (mask 0x0F) is the zero-indexed counter (ith packet of n).
  certificate_id : array
    The last 4 bytes of the certificate's SHA-1 fingerprint
  flags : int
  certificate_bytes : array
    DER encoded x.509 ECDSA certificate bytes
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'n_msg' / construct.Int8ul,
                   'certificate_id' / construct.Array(4, construct.Int8ul),
                   'flags' / construct.Int8ul,
                   'certificate_bytes' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'n_msg',
               'certificate_id',
               'flags',
               'certificate_bytes',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEcdsaCertificate,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEcdsaCertificate, self).__init__()
      self.msg_type = SBP_MSG_ECDSA_CERTIFICATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.n_msg = kwargs.pop('n_msg')
      self.certificate_id = kwargs.pop('certificate_id')
      self.flags = kwargs.pop('flags')
      self.certificate_bytes = kwargs.pop('certificate_bytes')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEcdsaCertificate.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEcdsaCertificate(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEcdsaCertificate._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEcdsaCertificate._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ECDSA CERTIFICATE"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEcdsaCertificate._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEcdsaCertificate, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_CERTIFICATE_CHAIN = 0x0C09
class MsgCertificateChain(SBP):
  """SBP class for message MSG_CERTIFICATE_CHAIN (0x0C09).

  You can have MSG_CERTIFICATE_CHAIN inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  root_certificate : array
    SHA-1 fingerprint of the root certificate
  intermediate_certificate : array
    SHA-1 fingerprint of the intermediate certificate
  corrections_certificate : array
    SHA-1 fingerprint of the corrections certificate
  expiration : UtcTime
    The time after which the signature given is no longer valid. Implementors
    should consult a time source (such as GNSS) to check if the current time
    is later than the expiration time, if the condition is true, signatures in
    the stream should not be considered valid.
  signature : ECDSASignature
    Signature (created by the root certificate) over the concatenation of the
    SBP payload bytes preceding this field. That is, the concatenation of
    `root_certificate`, `intermediate_certificate`, `corrections_certificate`
    and `expiration`.  This certificate chain (allow list) can also be
    validated by fetching it from `http(s)://certs.swiftnav.com/chain`.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'root_certificate' / construct.Array(20, construct.Int8ul),
                   'intermediate_certificate' / construct.Array(20, construct.Int8ul),
                   'corrections_certificate' / construct.Array(20, construct.Int8ul),
                   'expiration' / UtcTime._parser,
                   'signature' / ECDSASignature._parser,)
  __slots__ = [
               'root_certificate',
               'intermediate_certificate',
               'corrections_certificate',
               'expiration',
               'signature',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgCertificateChain,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgCertificateChain, self).__init__()
      self.msg_type = SBP_MSG_CERTIFICATE_CHAIN
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.root_certificate = kwargs.pop('root_certificate')
      self.intermediate_certificate = kwargs.pop('intermediate_certificate')
      self.corrections_certificate = kwargs.pop('corrections_certificate')
      self.expiration = kwargs.pop('expiration')
      self.signature = kwargs.pop('signature')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgCertificateChain.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgCertificateChain(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgCertificateChain._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgCertificateChain._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "CERTIFICATE CHAIN"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgCertificateChain._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgCertificateChain, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_CERTIFICATE_CHAIN_DEP = 0x0C05
class MsgCertificateChainDep(SBP):
  """SBP class for message MSG_CERTIFICATE_CHAIN_DEP (0x0C05).

  You can have MSG_CERTIFICATE_CHAIN_DEP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  root_certificate : array
    SHA-1 fingerprint of the root certificate
  intermediate_certificate : array
    SHA-1 fingerprint of the intermediate certificate
  corrections_certificate : array
    SHA-1 fingerprint of the corrections certificate
  expiration : UtcTime
    The certificate chain comprised of three fingerprints: root certificate,
    intermediate certificate and corrections certificate.
  signature : array
    An ECDSA signature (created by the root certificate) over the
    concatenation of the SBP payload bytes preceding this field. That is, the
    concatenation of `root_certificate`, `intermediate_certificate`,
    `corrections_certificate` and `expiration`.  This certificate chain (allow
    list) can also be validated by fetching it from
    `http(s)://certs.swiftnav.com/chain`.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'root_certificate' / construct.Array(20, construct.Int8ul),
                   'intermediate_certificate' / construct.Array(20, construct.Int8ul),
                   'corrections_certificate' / construct.Array(20, construct.Int8ul),
                   'expiration' / UtcTime._parser,
                   'signature' / construct.Array(64, construct.Int8ul),)
  __slots__ = [
               'root_certificate',
               'intermediate_certificate',
               'corrections_certificate',
               'expiration',
               'signature',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgCertificateChainDep,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgCertificateChainDep, self).__init__()
      self.msg_type = SBP_MSG_CERTIFICATE_CHAIN_DEP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.root_certificate = kwargs.pop('root_certificate')
      self.intermediate_certificate = kwargs.pop('intermediate_certificate')
      self.corrections_certificate = kwargs.pop('corrections_certificate')
      self.expiration = kwargs.pop('expiration')
      self.signature = kwargs.pop('signature')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgCertificateChainDep.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgCertificateChainDep(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgCertificateChainDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgCertificateChainDep._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "CERTIFICATE CHAIN DEP"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgCertificateChainDep._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgCertificateChainDep, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ECDSA_SIGNATURE = 0x0C08
class MsgEcdsaSignature(SBP):
  """SBP class for message MSG_ECDSA_SIGNATURE (0x0C08).

  You can have MSG_ECDSA_SIGNATURE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  An ECDSA-256 signature using SHA-256 as the message digest algorithm.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Describes the format of the `signed\_messages` field below.
  stream_counter : int
    Signature message counter. Zero indexed and incremented with each
    signature message.  The counter will not increment if this message was in
    response to an on demand request.  The counter will roll over after 256
    messages. Upon connection, the value of the counter may not initially be
    zero.
  on_demand_counter : int
    On demand message counter. Zero indexed and incremented with each
    signature message sent in response to an on demand message. The counter
    will roll over after 256 messages.  Upon connection, the value of the
    counter may not initially be zero.
  certificate_id : array
    The last 4 bytes of the certificate's SHA-1 fingerprint
  signature : ECDSASignature
    Signature over the frames of this message group.
  signed_messages : array
    CRCs of the messages covered by this signature.  For Skylark, which
    delivers SBP messages wrapped in Swift's proprietary RTCM message, these
    are the 24-bit CRCs from the RTCM message framing. For SBP only streams,
    this will be 16-bit CRCs from the SBP framing.  See the `flags` field to
    determine the type of CRCs covered.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int8ul,
                   'stream_counter' / construct.Int8ul,
                   'on_demand_counter' / construct.Int8ul,
                   'certificate_id' / construct.Array(4, construct.Int8ul),
                   'signature' / ECDSASignature._parser,
                   'signed_messages' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'flags',
               'stream_counter',
               'on_demand_counter',
               'certificate_id',
               'signature',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEcdsaSignature,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEcdsaSignature, self).__init__()
      self.msg_type = SBP_MSG_ECDSA_SIGNATURE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')
      self.stream_counter = kwargs.pop('stream_counter')
      self.on_demand_counter = kwargs.pop('on_demand_counter')
      self.certificate_id = kwargs.pop('certificate_id')
      self.signature = kwargs.pop('signature')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEcdsaSignature.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEcdsaSignature(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEcdsaSignature._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEcdsaSignature._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ECDSA SIGNATURE"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEcdsaSignature._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEcdsaSignature, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ECDSA_SIGNATURE_DEP_B = 0x0C07
class MsgEcdsaSignatureDepB(SBP):
  """SBP class for message MSG_ECDSA_SIGNATURE_DEP_B (0x0C07).

  You can have MSG_ECDSA_SIGNATURE_DEP_B inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Describes the format of the `signed\_messages` field below.
  stream_counter : int
    Signature message counter. Zero indexed and incremented with each
    signature message.  The counter will not increment if this message was in
    response to an on demand request.  The counter will roll over after 256
    messages. Upon connection, the value of the counter may not initially be
    zero.
  on_demand_counter : int
    On demand message counter. Zero indexed and incremented with each
    signature message sent in response to an on demand message. The counter
    will roll over after 256 messages.  Upon connection, the value of the
    counter may not initially be zero.
  certificate_id : array
    The last 4 bytes of the certificate's SHA-1 fingerprint
  n_signature_bytes : int
    Number of bytes to use of the signature field.  The DER encoded signature
    has a maximum size of 72 bytes but can vary between 70 and 72 bytes in
    length.
  signature : array
    DER encoded ECDSA signature for the messages using SHA-256 as the digest
    algorithm.
  signed_messages : array
    CRCs of the messages covered by this signature.  For Skylark, which
    delivers SBP messages wrapped in Swift's proprietary RTCM message, these
    are the 24-bit CRCs from the RTCM message framing. For SBP only streams,
    this will be 16-bit CRCs from the SBP framing.  See the `flags` field to
    determine the type of CRCs covered.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int8ul,
                   'stream_counter' / construct.Int8ul,
                   'on_demand_counter' / construct.Int8ul,
                   'certificate_id' / construct.Array(4, construct.Int8ul),
                   'n_signature_bytes' / construct.Int8ul,
                   'signature' / construct.Array(72, construct.Int8ul),
                   'signed_messages' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'flags',
               'stream_counter',
               'on_demand_counter',
               'certificate_id',
               'n_signature_bytes',
               'signature',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEcdsaSignatureDepB,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEcdsaSignatureDepB, self).__init__()
      self.msg_type = SBP_MSG_ECDSA_SIGNATURE_DEP_B
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')
      self.stream_counter = kwargs.pop('stream_counter')
      self.on_demand_counter = kwargs.pop('on_demand_counter')
      self.certificate_id = kwargs.pop('certificate_id')
      self.n_signature_bytes = kwargs.pop('n_signature_bytes')
      self.signature = kwargs.pop('signature')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEcdsaSignatureDepB.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEcdsaSignatureDepB(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEcdsaSignatureDepB._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEcdsaSignatureDepB._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ECDSA SIGNATURE DEP B"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEcdsaSignatureDepB._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEcdsaSignatureDepB, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ECDSA_SIGNATURE_DEP_A = 0x0C06
class MsgEcdsaSignatureDepA(SBP):
  """SBP class for message MSG_ECDSA_SIGNATURE_DEP_A (0x0C06).

  You can have MSG_ECDSA_SIGNATURE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  flags : int
    Describes the format of the `signed\_messages` field below.
  stream_counter : int
    Signature message counter. Zero indexed and incremented with each
    signature message.  The counter will not increment if this message was in
    response to an on demand request.  The counter will roll over after 256
    messages. Upon connection, the value of the counter may not initially be
    zero.
  on_demand_counter : int
    On demand message counter. Zero indexed and incremented with each
    signature message sent in response to an on demand message. The counter
    will roll over after 256 messages.  Upon connection, the value of the
    counter may not initially be zero.
  certificate_id : array
    The last 4 bytes of the certificate's SHA-1 fingerprint
  signature : array
    ECDSA signature for the messages using SHA-256 as the digest algorithm.
  signed_messages : array
    CRCs of the messages covered by this signature.  For Skylark, which
    delivers SBP messages wrapped in Swift's proprietary RTCM message, these
    are the 24-bit CRCs from the RTCM message framing. For SBP only streams,
    this will be 16-bit CRCs from the SBP framing.  See the `flags` field to
    determine the type of CRCs covered.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'flags' / construct.Int8ul,
                   'stream_counter' / construct.Int8ul,
                   'on_demand_counter' / construct.Int8ul,
                   'certificate_id' / construct.Array(4, construct.Int8ul),
                   'signature' / construct.Array(64, construct.Int8ul),
                   'signed_messages' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'flags',
               'stream_counter',
               'on_demand_counter',
               'certificate_id',
               'signature',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEcdsaSignatureDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEcdsaSignatureDepA, self).__init__()
      self.msg_type = SBP_MSG_ECDSA_SIGNATURE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.flags = kwargs.pop('flags')
      self.stream_counter = kwargs.pop('stream_counter')
      self.on_demand_counter = kwargs.pop('on_demand_counter')
      self.certificate_id = kwargs.pop('certificate_id')
      self.signature = kwargs.pop('signature')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEcdsaSignatureDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEcdsaSignatureDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEcdsaSignatureDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEcdsaSignatureDepA._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ECDSA SIGNATURE DEP A"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEcdsaSignatureDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEcdsaSignatureDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ED25519_CERTIFICATE_DEP = 0x0C02
class MsgEd25519CertificateDep(SBP):
  """SBP class for message MSG_ED25519_CERTIFICATE_DEP (0x0C02).

  You can have MSG_ED25519_CERTIFICATE_DEP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  n_msg : int
    Total number messages that make up the certificate. First nibble is the
    size of the sequence (n), second nibble is the zero-indexed counter (ith
    packet of n)
  fingerprint : array
    SHA-1 fingerprint of the associated certificate.
  certificate_bytes : array
    ED25519 certificate bytes.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'n_msg' / construct.Int8ul,
                   'fingerprint' / construct.Array(20, construct.Int8ul),
                   'certificate_bytes' / construct.GreedyRange(construct.Int8ul),)
  __slots__ = [
               'n_msg',
               'fingerprint',
               'certificate_bytes',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEd25519CertificateDep,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEd25519CertificateDep, self).__init__()
      self.msg_type = SBP_MSG_ED25519_CERTIFICATE_DEP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.n_msg = kwargs.pop('n_msg')
      self.fingerprint = kwargs.pop('fingerprint')
      self.certificate_bytes = kwargs.pop('certificate_bytes')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEd25519CertificateDep.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEd25519CertificateDep(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEd25519CertificateDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEd25519CertificateDep._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ED25519 CERTIFICATE DEP"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEd25519CertificateDep._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEd25519CertificateDep, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ED25519_SIGNATURE_DEP_A = 0x0C01
class MsgEd25519SignatureDepA(SBP):
  """SBP class for message MSG_ED25519_SIGNATURE_DEP_A (0x0C01).

  You can have MSG_ED25519_SIGNATURE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  signature : array
    ED25519 signature for messages.
  fingerprint : array
    SHA-1 fingerprint of the associated certificate.
  signed_messages : array
    CRCs of signed messages.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'signature' / construct.Array(64, construct.Int8ul),
                   'fingerprint' / construct.Array(20, construct.Int8ul),
                   'signed_messages' / construct.GreedyRange(construct.Int32ul),)
  __slots__ = [
               'signature',
               'fingerprint',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEd25519SignatureDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEd25519SignatureDepA, self).__init__()
      self.msg_type = SBP_MSG_ED25519_SIGNATURE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.signature = kwargs.pop('signature')
      self.fingerprint = kwargs.pop('fingerprint')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEd25519SignatureDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEd25519SignatureDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEd25519SignatureDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEd25519SignatureDepA._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ED25519 SIGNATURE DEP A"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEd25519SignatureDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEd25519SignatureDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_ED25519_SIGNATURE_DEP_B = 0x0C03
class MsgEd25519SignatureDepB(SBP):
  """SBP class for message MSG_ED25519_SIGNATURE_DEP_B (0x0C03).

  You can have MSG_ED25519_SIGNATURE_DEP_B inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  stream_counter : int
    Signature message counter. Zero indexed and incremented with each
    signature message.  The counter will not increment if this message was in
    response to an on demand request.  The counter will roll over after 256
    messages. Upon connection, the value of the counter may not initially be
    zero.
  on_demand_counter : int
    On demand message counter. Zero indexed and incremented with each
    signature message sent in response to an on demand message. The counter
    will roll over after 256 messages.  Upon connection, the value of the
    counter may not initially be zero.
  signature : array
    ED25519 signature for messages.
  fingerprint : array
    SHA-1 fingerprint of the associated certificate.
  signed_messages : array
    CRCs of signed messages.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'stream_counter' / construct.Int8ul,
                   'on_demand_counter' / construct.Int8ul,
                   'signature' / construct.Array(64, construct.Int8ul),
                   'fingerprint' / construct.Array(20, construct.Int8ul),
                   'signed_messages' / construct.GreedyRange(construct.Int32ul),)
  __slots__ = [
               'stream_counter',
               'on_demand_counter',
               'signature',
               'fingerprint',
               'signed_messages',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgEd25519SignatureDepB,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgEd25519SignatureDepB, self).__init__()
      self.msg_type = SBP_MSG_ED25519_SIGNATURE_DEP_B
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.stream_counter = kwargs.pop('stream_counter')
      self.on_demand_counter = kwargs.pop('on_demand_counter')
      self.signature = kwargs.pop('signature')
      self.fingerprint = kwargs.pop('fingerprint')
      self.signed_messages = kwargs.pop('signed_messages')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgEd25519SignatureDepB.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgEd25519SignatureDepB(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgEd25519SignatureDepB._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgEd25519SignatureDepB._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "ED25519 SIGNATURE DEP B"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgEd25519SignatureDepB._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgEd25519SignatureDepB, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0C04: MsgEcdsaCertificate,
  0x0C09: MsgCertificateChain,
  0x0C05: MsgCertificateChainDep,
  0x0C08: MsgEcdsaSignature,
  0x0C07: MsgEcdsaSignatureDepB,
  0x0C06: MsgEcdsaSignatureDepA,
  0x0C02: MsgEd25519CertificateDep,
  0x0C01: MsgEd25519SignatureDepA,
  0x0C03: MsgEd25519SignatureDepB,
}


from __future__ import print_function

import sys

import io

import json
import struct
import binascii

from construct.core import StreamError

import sbp.msg
import sbp.table

from sbp.msg import SBP_PREAMBLE


DEFAULT_JSON='rapidjson'
JSON_CHOICES=['json', 'rapidjson']
try:
    import rapidjson
except ImportError:
    DEFAULT_JSON='json'
    JSON_CHOICES=['json']

try:
    memoryview
except NameError:
    memoryview = lambda x: x

SBP_HEADER_LEN = 6


def base_cl_options():
    import argparse
    parser = argparse.ArgumentParser(prog="sbp2json", description="Swift Navigation SBP to JSON parser")
    parser.add_argument('--mode', type=str, choices=JSON_CHOICES, default=DEFAULT_JSON,
                        help="the JSON serialization library to use, default: {}".format(DEFAULT_JSON))
    parser.add_argument('--include', nargs="+", type=int, default=[],
                        help="list of SBP message IDs to include, empty means all")
    parser.add_argument("--unbuffered", action="store_true",
                        help="disable buffering when reading data from input (slower)")
    parser.add_argument('file', nargs='?', metavar='FILE', type=argparse.FileType('rb'),
                        default=sys.stdin, help="the input file, stdin by default")

    group_json = parser.add_argument_group('json specific arguments')
    group_json.add_argument(
        "--sort-keys",
        action="store_true",
        help="Sort JSON log elements by keys")

    return parser


def get_args():
    """
    Get and parse arguments.
    """
    parser = base_cl_options()
    args = parser.parse_args()

    return args


# return the read and expected CRCs from 'buf'
def get_crcs(buf, payload_len):
    crc_read, = struct.unpack("<H", buf[SBP_HEADER_LEN + payload_len:SBP_HEADER_LEN + payload_len + 2])
    crc_expected = binascii.crc_hqx(buf[1:SBP_HEADER_LEN + payload_len], 0)
    return crc_read, crc_expected


def dump(args, res):
    if 'json' == args.mode:
        sys.stdout.write(json.dumps(res.to_json_dict(),
                                    allow_nan=False,
                                    sort_keys=args.sort_keys,
                                    separators=(',', ':')))
    elif 'rapidjson' == args.mode:
        sys.stdout.write(rapidjson.dumps(res.to_json_dict()))
    sys.stdout.write("\n")


# Generator which parses SBP messages from a file object.
# Messages are read one at a time from the stream.
def iter_messages_unbuffered(fp):
    buf = b''

    def read_bytes_until(target_len):
        nonlocal buf
        while len(buf) < target_len:
            read_bytes = fp.read(target_len - len(buf))
            if len(read_bytes) == 0:
                raise IOError
            buf += read_bytes

    while True:
        # read header
        try:
            read_bytes_until(SBP_HEADER_LEN)
        except IOError:
            return

        # buf now contains at least SBP_HEADER_LEN bytes

        preamble, msg_type, sender, payload_len = struct.unpack("<BHHB", buf[:SBP_HEADER_LEN])

        # check preamble
        if preamble != SBP_PREAMBLE:
            buf = buf[1:] # drop first byte
            continue

        # read payload and CRC
        try:
            read_bytes_until(SBP_HEADER_LEN + payload_len + 2)
        except IOError:
            return

        # check CRC
        crc_read, crc_expected = get_crcs(buf, payload_len)
        if crc_read == crc_expected:
            yield msg_type, sender, payload_len, buf[:SBP_HEADER_LEN + payload_len + 2], crc_read
            buf = buf[SBP_HEADER_LEN + payload_len + 3:] # drop message
        else:
            sys.stderr.write("CRC error: {} vs {} for msg type {}\n".format(crc_read, crc_expected, msg_type))
            buf = buf[1:] # drop first byte


# Generator which parses SBP messages from a file object.
# Data is read from the stream in 4096 byte chunks.
def iter_messages_buffered(fp):
    buf = memoryview(bytearray(4096))
    unconsumed_offset = 0
    read_offset = 0
    buffer_remaining = len(buf)
    while True:
        if buffer_remaining == 0:
            buf[0:(read_offset - unconsumed_offset)] = buf[unconsumed_offset:read_offset]
            read_offset = read_offset - unconsumed_offset
            unconsumed_offset = 0
            buffer_remaining = len(buf) - read_offset
        mv = buf[read_offset:]
        read_length = fp.readinto(mv)
        if read_length == 0:
            unconsumed = read_offset - unconsumed_offset
            if unconsumed != 0:
                sys.stderr.write("unconsumed: {}\n".format(unconsumed))
                sys.stderr.flush()
            break
        read_offset += read_length
        buffer_remaining -= read_length
        while True:
            bytes_available = read_offset - unconsumed_offset
            b = buf[unconsumed_offset:(unconsumed_offset + bytes_available)]

            if len(b) > 0 and b[0] != SBP_PREAMBLE:
                consumed = 1
            else:
                if len(b) < SBP_HEADER_LEN:
                    # insufficient data, keep retrying until enough data becomes
                    # available
                    consumed = 0
                else:
                    preamble, msg_type, sender, payload_len = struct.unpack("<BHHB", b[:SBP_HEADER_LEN])
                    if len(b) < SBP_HEADER_LEN + payload_len + 2:
                        # insufficient data, keep retrying until enough data becomes
                        # available
                        consumed = 0
                    else:
                        # check CRC
                        b = b[:SBP_HEADER_LEN + payload_len + 2]
                        crc_read, crc_expected = get_crcs(b, payload_len)
                        if crc_read == crc_expected:
                            yield msg_type, sender, payload_len, b, crc_read
                            consumed = SBP_HEADER_LEN + payload_len + 2
                        else:
                            sys.stderr.write("CRC error: {} vs {} for msg type {}\n".format(crc_read, crc_expected, msg_type))
                            consumed = 1

            if consumed == 0:
                break

            unconsumed_offset += consumed


def sbp_main(args):
    reader = io.open(args.file.fileno(), 'rb')
    include = set(args.include)
    iter_fn = iter_messages_unbuffered if args.unbuffered else iter_messages_buffered

    for msg_type, sender, payload_len, buf, crc_read in iter_fn(reader):
        msg_buf = buf[SBP_HEADER_LEN:SBP_HEADER_LEN + payload_len]
        if not include or msg_type in include:
            try:
                msg = sbp.msg.SBP(msg_type, sender, payload_len, msg_buf, crc_read)
                msg = sbp.table.dispatch(msg)
                dump(args, msg)
            except (StreamError, ValueError):
                pass


def module_main():
    args = get_args()
    if not args:
        sys.exit(1)
    try:
        sbp_main(args)
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
# Copyright (C) 2011-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.

from __future__ import absolute_import

try:
    from ._version import full_version as __version__
except ImportError:
    __version__ = "0.0.0"



#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Standardized profiling messages from Swift Navigation devices.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize

# Automatically generated from piksi/yaml/swiftnav/sbp/profiling.yaml with generate.py.
# Please do not hand edit!


SBP_MSG_MEASUREMENT_POINT = 0xCF00
class MsgMeasurementPoint(SBP):
  """SBP class for message MSG_MEASUREMENT_POINT (0xCF00).

  You can have MSG_MEASUREMENT_POINT inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Tracks execution time of certain code paths in specially built products.
  This message should only be expected and processed on the direction of
  Swift's engineering teams.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  total_time : int
    Total time spent in measurement point (microseconds)
  num_executions : int
    Number of times measurement point has executed
  min : int
    Minimum execution time (microseconds)
  max : int
    Maximum execution time (microseconds)
  return_addr : int
    Return address
  id : int
    Unique ID
  slice_time : int
    CPU slice time (milliseconds)
  line : int
    Line number
  func : string
    Function name
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'total_time' / construct.Int32ul,
                   'num_executions' / construct.Int16ul,
                   'min' / construct.Int32ul,
                   'max' / construct.Int32ul,
                   'return_addr' / construct.Int64ul,
                   'id' / construct.Int64ul,
                   'slice_time' / construct.Int64ul,
                   'line' / construct.Int16ul,
                   'func' / construct.GreedyBytes,)
  __slots__ = [
               'total_time',
               'num_executions',
               'min',
               'max',
               'return_addr',
               'id',
               'slice_time',
               'line',
               'func',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgMeasurementPoint,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgMeasurementPoint, self).__init__()
      self.msg_type = SBP_MSG_MEASUREMENT_POINT
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.total_time = kwargs.pop('total_time')
      self.num_executions = kwargs.pop('num_executions')
      self.min = kwargs.pop('min')
      self.max = kwargs.pop('max')
      self.return_addr = kwargs.pop('return_addr')
      self.id = kwargs.pop('id')
      self.slice_time = kwargs.pop('slice_time')
      self.line = kwargs.pop('line')
      self.func = kwargs.pop('func')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgMeasurementPoint.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgMeasurementPoint(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgMeasurementPoint._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgMeasurementPoint._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "MEAS POINT"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgMeasurementPoint._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgMeasurementPoint, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0xCF00: MsgMeasurementPoint,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Telemetry messages reported by Starling engine. The messages include various
byproducts of state estimation and other logic across Starling and are aimed
at efficient issue diagnostics.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize
from sbp.gnss import GnssSignal

# Automatically generated from piksi/yaml/swiftnav/sbp/telemetry.yaml with generate.py.
# Please do not hand edit!


class TelemetrySV(object):
  """TelemetrySV.
  
  
  Parameters
  ----------
  az : int
    Azimuth angle (range 0..179)
  el : int
    Elevation angle (range -90..90)
  availability_flags : int
    Observation availability at filter update
  pseudorange_residual : int
    Pseudorange observation residual
  phase_residual : int
    Carrier-phase or carrier-phase-derived observation residual
  outlier_flags : int
    Reports if observation is marked as an outlier and is excluded from the
    update
  ephemeris_flags : int
    Ephemeris metadata
  correction_flags : int
    Reserved
  sid : GnssSignal
    GNSS signal identifier (16 bit)

  """
  _parser = construct.Struct(
                     'az' / construct.Int8ul,
                     'el' / construct.Int8sl,
                     'availability_flags' / construct.Int8ul,
                     'pseudorange_residual' / construct.Int16sl,
                     'phase_residual' / construct.Int16sl,
                     'outlier_flags' / construct.Int8ul,
                     'ephemeris_flags' / construct.Int8ul,
                     'correction_flags' / construct.Int8ul,
                     'sid' / GnssSignal._parser,)
  __slots__ = [
               'az',
               'el',
               'availability_flags',
               'pseudorange_residual',
               'phase_residual',
               'outlier_flags',
               'ephemeris_flags',
               'correction_flags',
               'sid',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.az = kwargs.pop('az')
      self.el = kwargs.pop('el')
      self.availability_flags = kwargs.pop('availability_flags')
      self.pseudorange_residual = kwargs.pop('pseudorange_residual')
      self.phase_residual = kwargs.pop('phase_residual')
      self.outlier_flags = kwargs.pop('outlier_flags')
      self.ephemeris_flags = kwargs.pop('ephemeris_flags')
      self.correction_flags = kwargs.pop('correction_flags')
      self.sid = kwargs.pop('sid')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TelemetrySV._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
SBP_MSG_TEL_SV = 0x0120
class MsgTelSv(SBP):
  """SBP class for message MSG_TEL_SV (0x0120).

  You can have MSG_TEL_SV inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  This message includes telemetry pertinent to satellite signals available to
  Starling.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  wn : int
    GPS week number
  tow : int
    GPS Time of Week
  n_obs : int
    Total number of observations. First nibble is the size of the sequence
    (n), second nibble is the zero-indexed counter (ith packet of n)
  origin_flags : int
    Flags to identify the filter type from which the telemetry is reported
    from
  sv_tel : array
    Array of per-signal telemetry entries
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'wn' / construct.Int16ul,
                   'tow' / construct.Int32ul,
                   'n_obs' / construct.Int8ul,
                   'origin_flags' / construct.Int8ul,
                   'sv_tel' / construct.GreedyRange(TelemetrySV._parser),)
  __slots__ = [
               'wn',
               'tow',
               'n_obs',
               'origin_flags',
               'sv_tel',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTelSv,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTelSv, self).__init__()
      self.msg_type = SBP_MSG_TEL_SV
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.wn = kwargs.pop('wn')
      self.tow = kwargs.pop('tow')
      self.n_obs = kwargs.pop('n_obs')
      self.origin_flags = kwargs.pop('origin_flags')
      self.sv_tel = kwargs.pop('sv_tel')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTelSv.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTelSv(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTelSv._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTelSv._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TEL SV"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTelSv._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTelSv, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0120: MsgTelSv,
}


#!/usr/bin/env python
# Copyright (C) 2015-2021 Swift Navigation Inc.
# Contact: https://support.swiftnav.com
#
# This source is subject to the license found in the file 'LICENSE' which must
# be be distributed together with this source. All other rights reserved.
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
# EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.


"""
Satellite code and carrier-phase tracking messages from the device.
"""

import json

import construct

from sbp.msg import SBP, SENDER_ID
from sbp.utils import fmt_repr, exclude_fields, walk_json_dict, containerize
from sbp.gnss import CarrierPhase, GPSTime, GPSTimeDep, GnssSignal, GnssSignalDep

# Automatically generated from piksi/yaml/swiftnav/sbp/tracking.yaml with generate.py.
# Please do not hand edit!


class TrackingChannelState(object):
  """TrackingChannelState.
  
  Tracking channel state for a specific satellite signal and measured signal
  power.
  
  Parameters
  ----------
  sid : GnssSignal
    GNSS signal being tracked
  fcn : int
    Frequency channel number (GLONASS only)
  cn0 : int
    Carrier-to-Noise density.  Zero implies invalid cn0.

  """
  _parser = construct.Struct(
                     'sid' / GnssSignal._parser,
                     'fcn' / construct.Int8ul,
                     'cn0' / construct.Int8ul,)
  __slots__ = [
               'sid',
               'fcn',
               'cn0',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.sid = kwargs.pop('sid')
      self.fcn = kwargs.pop('fcn')
      self.cn0 = kwargs.pop('cn0')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TrackingChannelState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class MeasurementState(object):
  """MeasurementState.
  
  Measurement Engine tracking channel state for a specific satellite signal
  and measured signal power. The mesid field for Glonass can either carry the
  FCN as 100 + FCN where FCN is in [-7, +6] or the Slot ID (from 1 to 28).
  
  Parameters
  ----------
  mesid : GnssSignal
    Measurement Engine GNSS signal being tracked (carries either Glonass FCN
    or SLOT)
  cn0 : int
    Carrier-to-Noise density.  Zero implies invalid cn0.

  """
  _parser = construct.Struct(
                     'mesid' / GnssSignal._parser,
                     'cn0' / construct.Int8ul,)
  __slots__ = [
               'mesid',
               'cn0',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.mesid = kwargs.pop('mesid')
      self.cn0 = kwargs.pop('cn0')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = MeasurementState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class TrackingChannelCorrelation(object):
  """TrackingChannelCorrelation.
  
  Structure containing in-phase and quadrature correlation components.
  
  Parameters
  ----------
  I : int
    In-phase correlation
  Q : int
    Quadrature correlation

  """
  _parser = construct.Struct(
                     'I' / construct.Int16sl,
                     'Q' / construct.Int16sl,)
  __slots__ = [
               'I',
               'Q',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.I = kwargs.pop('I')
      self.Q = kwargs.pop('Q')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TrackingChannelCorrelation._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class TrackingChannelCorrelationDep(object):
  """TrackingChannelCorrelationDep.
  
  Structure containing in-phase and quadrature correlation components.
  
  Parameters
  ----------
  I : int
    In-phase correlation
  Q : int
    Quadrature correlation

  """
  _parser = construct.Struct(
                     'I' / construct.Int32sl,
                     'Q' / construct.Int32sl,)
  __slots__ = [
               'I',
               'Q',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.I = kwargs.pop('I')
      self.Q = kwargs.pop('Q')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TrackingChannelCorrelationDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class TrackingChannelStateDepA(object):
  """TrackingChannelStateDepA.
  
  Deprecated.
  
  Parameters
  ----------
  state : int
    Status of tracking channel
  prn : int
    PRN-1 being tracked
  cn0 : float
    Carrier-to-noise density

  """
  _parser = construct.Struct(
                     'state' / construct.Int8ul,
                     'prn' / construct.Int8ul,
                     'cn0' / construct.Float32l,)
  __slots__ = [
               'state',
               'prn',
               'cn0',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.state = kwargs.pop('state')
      self.prn = kwargs.pop('prn')
      self.cn0 = kwargs.pop('cn0')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TrackingChannelStateDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
class TrackingChannelStateDepB(object):
  """TrackingChannelStateDepB.
  
  Deprecated.
  
  Parameters
  ----------
  state : int
    Status of tracking channel
  sid : GnssSignalDep
    GNSS signal being tracked
  cn0 : float
    Carrier-to-noise density

  """
  _parser = construct.Struct(
                     'state' / construct.Int8ul,
                     'sid' / GnssSignalDep._parser,
                     'cn0' / construct.Float32l,)
  __slots__ = [
               'state',
               'sid',
               'cn0',
              ]

  def __init__(self, payload=None, **kwargs):
    if payload:
      self.from_binary(payload)
    else:
      self.state = kwargs.pop('state')
      self.sid = kwargs.pop('sid')
      self.cn0 = kwargs.pop('cn0')

  def __repr__(self):
    return fmt_repr(self)
  
  def from_binary(self, d):
    p = TrackingChannelStateDepB._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))
    
SBP_MSG_TRACKING_STATE_DETAILED_DEP_A = 0x0021
class MsgTrackingStateDetailedDepA(SBP):
  """SBP class for message MSG_TRACKING_STATE_DETAILED_DEP_A (0x0021).

  You can have MSG_TRACKING_STATE_DETAILED_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The tracking message returns a set tracking channel parameters for a single
  tracking channel useful for debugging issues.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  recv_time : int
    Receiver clock time.
  tot : GPSTime
    Time of transmission of signal from satellite. TOW only valid when TOW
    status is decoded or propagated. WN only valid when week number valid flag
    is set.
  P : int
    Pseudorange observation. Valid only when pseudorange valid flag is set.
  P_std : int
    Pseudorange observation standard deviation. Valid only when pseudorange
    valid flag is set.
  L : CarrierPhase
    Carrier phase observation with typical sign convention. Valid only when
    PLL pessimistic lock is achieved.
  cn0 : int
    Carrier-to-Noise density
  lock : int
    Lock time. It is encoded according to DF402 from the RTCM 10403.2
    Amendment 2 specification. Valid values range from 0 to 15.
  sid : GnssSignal
    GNSS signal identifier.
  doppler : int
    Carrier Doppler frequency.
  doppler_std : int
    Carrier Doppler frequency standard deviation.
  uptime : int
    Number of seconds of continuous tracking. Specifies how much time signal
    is in continuous track.
  clock_offset : int
    TCXO clock offset. Valid only when valid clock valid flag is set.
  clock_drift : int
    TCXO clock drift. Valid only when valid clock valid flag is set.
  corr_spacing : int
    Early-Prompt (EP) and Prompt-Late (PL) correlators spacing.
  acceleration : int
    Acceleration. Valid only when acceleration valid flag is set.
  sync_flags : int
    Synchronization status flags.
  tow_flags : int
    TOW status flags.
  track_flags : int
    Tracking loop status flags.
  nav_flags : int
    Navigation data status flags.
  pset_flags : int
    Parameters sets flags.
  misc_flags : int
    Miscellaneous flags.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'recv_time' / construct.Int64ul,
                   'tot' / GPSTime._parser,
                   'P' / construct.Int32ul,
                   'P_std' / construct.Int16ul,
                   'L' / CarrierPhase._parser,
                   'cn0' / construct.Int8ul,
                   'lock' / construct.Int16ul,
                   'sid' / GnssSignal._parser,
                   'doppler' / construct.Int32sl,
                   'doppler_std' / construct.Int16ul,
                   'uptime' / construct.Int32ul,
                   'clock_offset' / construct.Int16sl,
                   'clock_drift' / construct.Int16sl,
                   'corr_spacing' / construct.Int16ul,
                   'acceleration' / construct.Int8sl,
                   'sync_flags' / construct.Int8ul,
                   'tow_flags' / construct.Int8ul,
                   'track_flags' / construct.Int8ul,
                   'nav_flags' / construct.Int8ul,
                   'pset_flags' / construct.Int8ul,
                   'misc_flags' / construct.Int8ul,)
  __slots__ = [
               'recv_time',
               'tot',
               'P',
               'P_std',
               'L',
               'cn0',
               'lock',
               'sid',
               'doppler',
               'doppler_std',
               'uptime',
               'clock_offset',
               'clock_drift',
               'corr_spacing',
               'acceleration',
               'sync_flags',
               'tow_flags',
               'track_flags',
               'nav_flags',
               'pset_flags',
               'misc_flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingStateDetailedDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingStateDetailedDepA, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_STATE_DETAILED_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.recv_time = kwargs.pop('recv_time')
      self.tot = kwargs.pop('tot')
      self.P = kwargs.pop('P')
      self.P_std = kwargs.pop('P_std')
      self.L = kwargs.pop('L')
      self.cn0 = kwargs.pop('cn0')
      self.lock = kwargs.pop('lock')
      self.sid = kwargs.pop('sid')
      self.doppler = kwargs.pop('doppler')
      self.doppler_std = kwargs.pop('doppler_std')
      self.uptime = kwargs.pop('uptime')
      self.clock_offset = kwargs.pop('clock_offset')
      self.clock_drift = kwargs.pop('clock_drift')
      self.corr_spacing = kwargs.pop('corr_spacing')
      self.acceleration = kwargs.pop('acceleration')
      self.sync_flags = kwargs.pop('sync_flags')
      self.tow_flags = kwargs.pop('tow_flags')
      self.track_flags = kwargs.pop('track_flags')
      self.nav_flags = kwargs.pop('nav_flags')
      self.pset_flags = kwargs.pop('pset_flags')
      self.misc_flags = kwargs.pop('misc_flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingStateDetailedDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingStateDetailedDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingStateDetailedDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingStateDetailedDepA._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK STATE DETAILED DEP A"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingStateDetailedDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingStateDetailedDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_STATE_DETAILED_DEP = 0x0011
class MsgTrackingStateDetailedDep(SBP):
  """SBP class for message MSG_TRACKING_STATE_DETAILED_DEP (0x0011).

  You can have MSG_TRACKING_STATE_DETAILED_DEP inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  recv_time : int
    Receiver clock time.
  tot : GPSTimeDep
    Time of transmission of signal from satellite. TOW only valid when TOW
    status is decoded or propagated. WN only valid when week number valid flag
    is set.
  P : int
    Pseudorange observation. Valid only when pseudorange valid flag is set.
  P_std : int
    Pseudorange observation standard deviation. Valid only when pseudorange
    valid flag is set.
  L : CarrierPhase
    Carrier phase observation with typical sign convention. Valid only when
    PLL pessimistic lock is achieved.
  cn0 : int
    Carrier-to-Noise density
  lock : int
    Lock time. It is encoded according to DF402 from the RTCM 10403.2
    Amendment 2 specification. Valid values range from 0 to 15.
  sid : GnssSignalDep
    GNSS signal identifier.
  doppler : int
    Carrier Doppler frequency.
  doppler_std : int
    Carrier Doppler frequency standard deviation.
  uptime : int
    Number of seconds of continuous tracking. Specifies how much time signal
    is in continuous track.
  clock_offset : int
    TCXO clock offset. Valid only when valid clock valid flag is set.
  clock_drift : int
    TCXO clock drift. Valid only when valid clock valid flag is set.
  corr_spacing : int
    Early-Prompt (EP) and Prompt-Late (PL) correlators spacing.
  acceleration : int
    Acceleration. Valid only when acceleration valid flag is set.
  sync_flags : int
    Synchronization status flags.
  tow_flags : int
    TOW status flags.
  track_flags : int
    Tracking loop status flags.
  nav_flags : int
    Navigation data status flags.
  pset_flags : int
    Parameters sets flags.
  misc_flags : int
    Miscellaneous flags.
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'recv_time' / construct.Int64ul,
                   'tot' / GPSTimeDep._parser,
                   'P' / construct.Int32ul,
                   'P_std' / construct.Int16ul,
                   'L' / CarrierPhase._parser,
                   'cn0' / construct.Int8ul,
                   'lock' / construct.Int16ul,
                   'sid' / GnssSignalDep._parser,
                   'doppler' / construct.Int32sl,
                   'doppler_std' / construct.Int16ul,
                   'uptime' / construct.Int32ul,
                   'clock_offset' / construct.Int16sl,
                   'clock_drift' / construct.Int16sl,
                   'corr_spacing' / construct.Int16ul,
                   'acceleration' / construct.Int8sl,
                   'sync_flags' / construct.Int8ul,
                   'tow_flags' / construct.Int8ul,
                   'track_flags' / construct.Int8ul,
                   'nav_flags' / construct.Int8ul,
                   'pset_flags' / construct.Int8ul,
                   'misc_flags' / construct.Int8ul,)
  __slots__ = [
               'recv_time',
               'tot',
               'P',
               'P_std',
               'L',
               'cn0',
               'lock',
               'sid',
               'doppler',
               'doppler_std',
               'uptime',
               'clock_offset',
               'clock_drift',
               'corr_spacing',
               'acceleration',
               'sync_flags',
               'tow_flags',
               'track_flags',
               'nav_flags',
               'pset_flags',
               'misc_flags',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingStateDetailedDep,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingStateDetailedDep, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_STATE_DETAILED_DEP
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.recv_time = kwargs.pop('recv_time')
      self.tot = kwargs.pop('tot')
      self.P = kwargs.pop('P')
      self.P_std = kwargs.pop('P_std')
      self.L = kwargs.pop('L')
      self.cn0 = kwargs.pop('cn0')
      self.lock = kwargs.pop('lock')
      self.sid = kwargs.pop('sid')
      self.doppler = kwargs.pop('doppler')
      self.doppler_std = kwargs.pop('doppler_std')
      self.uptime = kwargs.pop('uptime')
      self.clock_offset = kwargs.pop('clock_offset')
      self.clock_drift = kwargs.pop('clock_drift')
      self.corr_spacing = kwargs.pop('corr_spacing')
      self.acceleration = kwargs.pop('acceleration')
      self.sync_flags = kwargs.pop('sync_flags')
      self.tow_flags = kwargs.pop('tow_flags')
      self.track_flags = kwargs.pop('track_flags')
      self.nav_flags = kwargs.pop('nav_flags')
      self.pset_flags = kwargs.pop('pset_flags')
      self.misc_flags = kwargs.pop('misc_flags')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingStateDetailedDep.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingStateDetailedDep(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingStateDetailedDep._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingStateDetailedDep._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK STATE DETAILED DEP"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingStateDetailedDep._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingStateDetailedDep, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_STATE = 0x0041
class MsgTrackingState(SBP):
  """SBP class for message MSG_TRACKING_STATE (0x0041).

  You can have MSG_TRACKING_STATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The tracking message returns a variable-length array of tracking channel
  states. It reports status and carrier-to-noise density measurements for all
  tracked satellites.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  states : array
    Signal tracking channel state
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'states' / construct.GreedyRange(TrackingChannelState._parser),)
  __slots__ = [
               'states',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingState,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingState, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_STATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.states = kwargs.pop('states')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingState.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingState(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingState._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK STATE"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingState._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingState, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_MEASUREMENT_STATE = 0x0061
class MsgMeasurementState(SBP):
  """SBP class for message MSG_MEASUREMENT_STATE (0x0061).

  You can have MSG_MEASUREMENT_STATE inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  The tracking message returns a variable-length array of tracking channel
  states. It reports status and carrier-to-noise density measurements for all
  tracked satellites.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  states : array
    ME signal tracking channel state
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'states' / construct.GreedyRange(MeasurementState._parser),)
  __slots__ = [
               'states',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgMeasurementState,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgMeasurementState, self).__init__()
      self.msg_type = SBP_MSG_MEASUREMENT_STATE
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.states = kwargs.pop('states')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgMeasurementState.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgMeasurementState(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgMeasurementState._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgMeasurementState._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "MEAS STATE"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgMeasurementState._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgMeasurementState, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_IQ = 0x002D
class MsgTrackingIq(SBP):
  """SBP class for message MSG_TRACKING_IQ (0x002D).

  You can have MSG_TRACKING_IQ inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  When enabled, a tracking channel can output the correlations at each update
  interval.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  channel : int
    Tracking channel of origin
  sid : GnssSignal
    GNSS signal identifier
  corrs : array
    Early, Prompt and Late correlations
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'channel' / construct.Int8ul,
                   'sid' / GnssSignal._parser,
                   'corrs' / construct.Array(3, TrackingChannelCorrelation._parser),)
  __slots__ = [
               'channel',
               'sid',
               'corrs',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingIq,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingIq, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_IQ
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.channel = kwargs.pop('channel')
      self.sid = kwargs.pop('sid')
      self.corrs = kwargs.pop('corrs')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingIq.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingIq(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingIq._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingIq._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK IQ"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingIq._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingIq, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_IQ_DEP_B = 0x002C
class MsgTrackingIqDepB(SBP):
  """SBP class for message MSG_TRACKING_IQ_DEP_B (0x002C).

  You can have MSG_TRACKING_IQ_DEP_B inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  channel : int
    Tracking channel of origin
  sid : GnssSignal
    GNSS signal identifier
  corrs : array
    Early, Prompt and Late correlations
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'channel' / construct.Int8ul,
                   'sid' / GnssSignal._parser,
                   'corrs' / construct.Array(3, TrackingChannelCorrelationDep._parser),)
  __slots__ = [
               'channel',
               'sid',
               'corrs',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingIqDepB,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingIqDepB, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_IQ_DEP_B
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.channel = kwargs.pop('channel')
      self.sid = kwargs.pop('sid')
      self.corrs = kwargs.pop('corrs')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingIqDepB.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingIqDepB(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingIqDepB._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingIqDepB._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK IQ DEP B"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingIqDepB._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingIqDepB, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_IQ_DEP_A = 0x001C
class MsgTrackingIqDepA(SBP):
  """SBP class for message MSG_TRACKING_IQ_DEP_A (0x001C).

  You can have MSG_TRACKING_IQ_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  channel : int
    Tracking channel of origin
  sid : GnssSignalDep
    GNSS signal identifier
  corrs : array
    Early, Prompt and Late correlations
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'channel' / construct.Int8ul,
                   'sid' / GnssSignalDep._parser,
                   'corrs' / construct.Array(3, TrackingChannelCorrelationDep._parser),)
  __slots__ = [
               'channel',
               'sid',
               'corrs',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingIqDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingIqDepA, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_IQ_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.channel = kwargs.pop('channel')
      self.sid = kwargs.pop('sid')
      self.corrs = kwargs.pop('corrs')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingIqDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingIqDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingIqDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingIqDepA._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK IQ DEP A"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingIqDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingIqDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_STATE_DEP_A = 0x0016
class MsgTrackingStateDepA(SBP):
  """SBP class for message MSG_TRACKING_STATE_DEP_A (0x0016).

  You can have MSG_TRACKING_STATE_DEP_A inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  states : array
    Satellite tracking channel state
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'states' / construct.GreedyRange(TrackingChannelStateDepA._parser),)
  __slots__ = [
               'states',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingStateDepA,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingStateDepA, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_STATE_DEP_A
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.states = kwargs.pop('states')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingStateDepA.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingStateDepA(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingStateDepA._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingStateDepA._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK STATE DEP A"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingStateDepA._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingStateDepA, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    
SBP_MSG_TRACKING_STATE_DEP_B = 0x0013
class MsgTrackingStateDepB(SBP):
  """SBP class for message MSG_TRACKING_STATE_DEP_B (0x0013).

  You can have MSG_TRACKING_STATE_DEP_B inherit its fields directly
  from an inherited SBP object, or construct it inline using a dict
  of its fields.

  
  Deprecated.

  Parameters
  ----------
  sbp : SBP
    SBP parent object to inherit from.
  states : array
    Signal tracking channel state
  sender : int
    Optional sender ID, defaults to SENDER_ID (see sbp/msg.py).

  """
  _parser = construct.Struct(
                   'states' / construct.GreedyRange(TrackingChannelStateDepB._parser),)
  __slots__ = [
               'states',
              ]

  def __init__(self, sbp=None, **kwargs):
    if sbp:
      super( MsgTrackingStateDepB,
             self).__init__(sbp.msg_type, sbp.sender, sbp.length,
                            sbp.payload, sbp.crc)
      self.from_binary(sbp.payload)
    else:
      super( MsgTrackingStateDepB, self).__init__()
      self.msg_type = SBP_MSG_TRACKING_STATE_DEP_B
      self.sender = kwargs.pop('sender', SENDER_ID)
      self.states = kwargs.pop('states')

  def __repr__(self):
    return fmt_repr(self)

  @staticmethod
  def from_json(s):
    """Given a JSON-encoded string s, build a message object.

    """
    d = json.loads(s)
    return MsgTrackingStateDepB.from_json_dict(d)

  @staticmethod
  def from_json_dict(d):
    sbp = SBP.from_json_dict(d)
    return MsgTrackingStateDepB(sbp, **d)

 
  def from_binary(self, d):
    """Given a binary payload d, update the appropriate payload fields of
    the message.

    """
    p = MsgTrackingStateDepB._parser.parse(d)
    for n in self.__class__.__slots__:
      setattr(self, n, getattr(p, n))

  def to_binary(self):
    """Produce a framed/packed SBP message.

    """
    c = containerize(exclude_fields(self))
    self.payload = MsgTrackingStateDepB._parser.build(c)
    return self.pack()

  def friendly_name(self):
    """Produces friendly human-readable name for this message

    """
    return "TRK STATE DEP B"

  def into_buffer(self, buf, offset):
    """Produce a framed/packed SBP message into the provided buffer and offset.

    """
    self.payload = containerize(exclude_fields(self))
    self.parser = MsgTrackingStateDepB._parser
    self.stream_payload.reset(buf, offset)
    return self.pack_into(buf, offset, self._build_payload)

  def to_json_dict(self):
    self.to_binary()
    d = super( MsgTrackingStateDepB, self).to_json_dict()
    j = walk_json_dict(exclude_fields(self))
    d.update(j)
    return d
    

msg_classes = {
  0x0021: MsgTrackingStateDetailedDepA,
  0x0011: MsgTrackingStateDetailedDep,
  0x0041: MsgTrackingState,
  0x0061: MsgMeasurementState,
  0x002D: MsgTrackingIq,
  0x002C: MsgTrackingIqDepB,
  0x001C: MsgTrackingIqDepA,
  0x0016: MsgTrackingStateDepA,
  0x0013: MsgTrackingStateDepB,
}